/**
 * @brief Este projeto demonstra o cdigo (binrio) ASCII usado para representar os
 * caracteres alfa-numericos
 * @author Wu Shin-Ting
 * @date 13/10/2020
 */

#include <string.h> 	//declaracao das funcoes de string
#include "derivative.h" /* include peripheral declarations */

#define GPIO_PIN(x) (1<<x)

/**
 * @brief atraso em numero de interacoes <i>
 * @param[in] i numero de iteracoes
 */
void delay (unsigned int i)
{
	while (i) i--;
}

/**
 * @brief Inicializa os pinos conectados ao latch 74573 onde estao conectados os 8 leds vermelhos
 */
void GPIO_initLatch74573(){
	SIM_SCGC5 |= SIM_SCGC5_PORTC_MASK ;     ///< Habilita o clock da PORTA C

	// Configura os pinos PORTC[10:0] como GPIO de saida 
	PORTC_PCR0 |= PORT_PCR_MUX(0x1);       ///< D0-D7 dos dados (GPIO)
	PORTC_PCR1 |= PORT_PCR_MUX(0x1);
	PORTC_PCR2 |= PORT_PCR_MUX(0x1);
	PORTC_PCR3 |= PORT_PCR_MUX(0x1);
	PORTC_PCR4 |= PORT_PCR_MUX(0x1);
	PORTC_PCR5 |= PORT_PCR_MUX(0x1);
	PORTC_PCR6 |= PORT_PCR_MUX(0x1);
	PORTC_PCR7 |= PORT_PCR_MUX(0x1);
	
	PORTC_PCR10 |= PORT_PCR_MUX(0x1);       ///< /LE do latch

	GPIOC_PDDR |= GPIO_PDDR_PDD(GPIO_PIN(10) |GPIO_PIN(7) | 
			GPIO_PIN(6) | GPIO_PIN(5) | GPIO_PIN(4) | GPIO_PIN(3) | 
			GPIO_PIN(2) | GPIO_PIN(1) | GPIO_PIN(0));       ///< Configura como saidas
	GPIOC_PDOR &= ~GPIO_PDOR_PDO(GPIO_PIN(10) |GPIO_PIN(7) | 
			GPIO_PIN(6) | GPIO_PIN(5) | GPIO_PIN(4) | GPIO_PIN(3) | 
			GPIO_PIN(2) | GPIO_PIN(1) | GPIO_PIN(0)); ///< Inicializa os pinos com 0
}

/**
 * @brief Escrever nos pinos de dados um caractere de 8 bits
 * @param[in] c caractere
 */
void GPIO_setByte(char c) {
	// colocar os dados nos pinos de dados DO-D7
	GPIOC_PDOR &= 0xFFFFFF00; 		// zerar o byte menos significativo
	GPIOC_PDOR |= (unsigned int) c; // setas os bits em 1 no byte menos significativo
	
}

/**
 * @brief Gerar um pulso de habilitao de ~50ns
 */
void GPIO_enableLE()
{
	// gerar um pulso LE (> 15 ns)
	GPIOC_PSOR = GPIO_PIN (10);		// setar em 1 (1 ciclo de instruo ~50ns) 
	GPIOC_PCOR = GPIO_PIN (10);		// resetar em 0
}

/**
 * @brief Transferir um byte ao latch
 * @param[in] c byte
 */
void escreveLatch(char c)
{
	GPIO_setByte(c);
	GPIO_enableLE();
}

int main(void)
{
	char str3[38], *end_str;
	char digitos[]="0123456789";
	char maiusculas[] = "ABCDEFGHIJK";
	char minusculas[] = "abcdefghijk";
	char vermelho[]= "VERMELHO";     //8 caracteres
	char verde[] = "VERDE";	  //5 caracteres
	char azul[] = "AZUL";		  //4 caracteres
	char amarelo[] = "AMARELO";	  //7 caracteres
	char ciano = "CIANO";	  //5 caracteres
	char magenta[] = "MAGENTA";	  //7 caracteres
	
	uint8_t j, len;
	
	GPIO_initLatch74573();
	
	for(;;) {	   
		// mostrar ASCII dos digitos
		for (j=0; j<10; j++) { 
			escreveLatch(digitos[j]);
			delay (1500000);
		}
		
		// mostrar ASCII das maisculas
		j=0;
		while (maiusculas[j]) { 
			escreveLatch(maiusculas[j]);
			j++;
			delay (1500000);
		}
		
		// mostrar ASCII das minusculas
		j=0;
		while (minusculas[j]) { 
			escreveLatch(minusculas[j]);
			j++;
			delay (1500000);
		}
		
		//copiar a terceira cor em str3
		strcpy (str3, azul);
		len = (uint8_t) strlen (str3);
		for (j=0; j< len; j++) {
			escreveLatch(str3[j]);
			delay (1500000);
		}
		
		//limpar os leds vermelhos
		escreveLatch(0x00);
		delay (1500000);
		
		//concatenar a str3 a quinta cor
	   	strcat (str3, amarelo);
	   	len = (uint8_t) strlen (str3);
	   	j = 0;
		while (str3[j] != '\0') {
			escreveLatch(*(str3+j));
			delay (1500000);
			j++;
		}
	   	
		//limpar os leds vermelhos
		escreveLatch(0x00);
		delay (1500000);

		//obter o endereco de 'G' em "MAGENTA"
		end_str = strchr (magenta, 'G');
		
		//concatenar 3 caracteres de end_end a str3
	   	strncat (str3, end_str, 3);
	   	len = (uint8_t) strlen (str3);
	   	j = 0;
		do {
			escreveLatch(*(str3+j));
			delay (1500000);
			j++;
		} while (j < (len-1));
	}
	
	return 0;
}
